package de.mdelab.mlsdm.interpreter.debug;

import java.io.OutputStream;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EClassifier;

import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityParameter;
import de.mdelab.mlsdm.ActivityParameterDirectionEnum;
import de.mdelab.mlsdm.interpreter.MLSDMInterpreter;
import de.mdelab.mlsdm.interpreter.coverage.CoverageReport;
import de.mdelab.mlsdm.interpreter.coverage.MLSDMCoverageNotificationReceiver;
import de.mdelab.mlsdm.interpreter.eclipse.MLSDMEclipseInterpreter;
import de.mdelab.mlsdm.interpreter.executionTrace.MLSDMExecutionTraceNotificationReceiver;
import de.mdelab.mlsdm.interpreter.notifications.MLSDMOutputStreamNotificationReceiver;
import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.SDMInterpreter;
import de.mdelab.sdm.interpreter.core.debug.SDCoverageNotificationReceiver;
import de.mdelab.sdm.interpreter.core.debug.SDDebugger;
import de.mdelab.sdm.interpreter.core.debug.SDEclipseInterpreterLauncher;
import de.mdelab.sdm.interpreter.core.debug.SDExecution;
import de.mdelab.sdm.interpreter.core.executionTrace.ExecutionTrace;
import de.mdelab.sdm.interpreter.core.executionTrace.ExecutionTraceNotificationReceiver;
import de.mdelab.sdm.interpreter.core.notifications.OutputStreamNotificationReceiver;
import de.mdelab.sdm.interpreter.core.variables.Variable;

public class MLSDMInterpreterLauncher extends SDEclipseInterpreterLauncher<CoverageReport, Activity, EClassifier> {

	public MLSDMInterpreterLauncher() {
		super();
	}
	
	public MLSDMInterpreterLauncher( final ClassLoader p_classLoader ) {
		super( p_classLoader );
	}
	
	@Override
	public MLSDMExecution launch( 	final Activity p_activity,
									final List<Variable<EClassifier>> p_variableBindings,
									final Integer p_debuggerPort,
									final Integer p_resourceSetSynchronizerPort,
									final boolean pb_debugOutput,
									final boolean pb_execTrace,
									final boolean pb_coverageTrace,
									final IProgressMonitor p_monitor,
									final Logger p_logger,
									final OutputStream p_outputStream )
	throws SDMException {
		return (MLSDMExecution) super.launch( p_activity, p_variableBindings, p_debuggerPort, p_resourceSetSynchronizerPort, pb_debugOutput, pb_execTrace, pb_coverageTrace, p_monitor, p_logger, p_outputStream );
	}

	@Override
	protected SDMInterpreter<Activity, ?, ?, ?, ?, ?, EClassifier, ?, ?> createInterpreter()
	throws SDMException {
		 return new MLSDMEclipseInterpreter( classLoader );
	}

	@Override
	protected SDDebugger<?, Activity, ?, ?, ?> createDebugger( 	final SDMInterpreter<Activity, ?, ?, ?, ?, ?, EClassifier, ?, ?> p_sdi,
																final Integer p_debuggerPort, 
																final Integer p_resourceSyncPort) {
		return new MLSDMDebugger( (MLSDMInterpreter) p_sdi, p_debuggerPort, p_resourceSyncPort );
	}

	@Override
	protected ExecutionTraceNotificationReceiver<Activity, ?, ?, ?, ?, ?, EClassifier, ?, ?> createExecutionTraceNotificationReceiver() {
		return new MLSDMExecutionTraceNotificationReceiver();
	}

	@Override
	protected OutputStreamNotificationReceiver<Activity, ?, ?, ?, ?, ?, EClassifier, ?, ?> createOutputStreamNotificationReceiver( final OutputStream p_outputStream ) {
		return  new MLSDMOutputStreamNotificationReceiver( p_outputStream );
	}

	@Override
	protected SDCoverageNotificationReceiver<CoverageReport, EClassifier> createCoverageNotificationReceiver() {
		return new MLSDMCoverageNotificationReceiver();
	}

	@Override
	protected String getName( final Activity p_activity ) {
		assert p_activity != null;
		
		return p_activity.getName();
	}

	@Override
	protected void logOutputParameterValues(	final Activity p_activity,
												final Map<String, Variable<EClassifier>> p_returnVariables,
												final Logger p_logger ) {
		for ( final ActivityParameter param : p_activity.getParameters() ) {
			final ActivityParameterDirectionEnum direction = param.getDirection();
			
			if ( direction == ActivityParameterDirectionEnum.OUT	|| direction == ActivityParameterDirectionEnum.INOUT ) {
				p_logger.info( "Output parameter '"	+ param.getName() + "' : " + p_returnVariables.get( param.getName() ).getValue() );
			}
		}
	}

	@Override
	public MLSDMDebugger getDebugger() {
		return (MLSDMDebugger) super.getDebugger();
	}

	@Override
	protected SDExecution<CoverageReport, EClassifier> createExecutionResult( 	final ExecutionTrace p_execTrace,
																				final CoverageReport p_covReport,
																				final Map<String, Variable<EClassifier>> p_returnVariables ) {
		return new MLSDMExecution( p_execTrace, p_covReport, p_returnVariables );
	}
}
