package de.mdelab.mlsdm.interpreter.debug.ui.launcher;

import java.util.LinkedHashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.widgets.Composite;

import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityParameter;
import de.mdelab.mlsdm.ActivityParameterDirectionEnum;
import de.mdelab.mlsdm.MlsdmPackage;
import de.mdelab.mlsdm.interpreter.debug.ui.launcher.table.MLSDMActivityParameterTableViewer;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.ISDLaunchUIConstants;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.SDInterpreterLaunchMainTab;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.SDActivityParameterTableViewer;

public class MLSDMInterpreterLaunchMainTab extends SDInterpreterLaunchMainTab<Activity> {
	
	public static final String SD_FILE_EXT = MlsdmPackage.eNAME;

	@Override
	protected boolean validateActivityParameters( 	final ILaunchConfiguration p_configuration,
													final Activity p_activity )
	throws CoreException {
		final String errMessage = validateActivityParams( p_configuration, p_activity );
		
		if ( errMessage != null ) {
			setErrorMessage( errMessage );
			
			return false;
		}
		
		return true;
	}
	
	public static String validateActivityParams( 	final ILaunchConfiguration p_configuration,
													final Activity p_activity )
	throws CoreException {
		final Map<String, String> parameters = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_STORY_ACTIVITY_PARAMETERS, new LinkedHashMap<String, String>() );
		
		for (final ActivityParameter parameter : p_activity.getParameters() ) {
			if ( parameter.getDirection() == ActivityParameterDirectionEnum.IN || parameter.getDirection() == ActivityParameterDirectionEnum.INOUT ) {
				final String parameterName = parameter.getName();
				final String objUriStr = parameters.get( parameterName );
				
				if ( objUriStr == null || objUriStr.isEmpty() ) {
					return "The activity parameter '" + parameterName + "' is not specified.";
				}
				
				final EObject paramValue =  p_activity.eResource().getResourceSet().getEObject( URI.createURI( objUriStr) , true );
				
				if ( paramValue == null ) {
					return "The value for activity parameter '" + parameterName + "' is not specified.";
				}
				
				if ( !parameter.getType().isInstance( paramValue ) ) {
					return "The value for activity parameter '" + parameterName + "' is not an instance of " + parameter.getType().getName() + ".";
				}
			}
		}
		
		return null;
	}

	@Override
	protected SDActivityParameterTableViewer<Activity, ?, ?> createActivityParameterTableViewer( final Composite p_parent ) {
		return new MLSDMActivityParameterTableViewer( p_parent );
	}

	@Override
	protected String getSDFileExtension() {
		return SD_FILE_EXT;
	}

	@Override
	protected String getName( final Activity p_activity ) {
		assert p_activity != null;
		
		return p_activity.getName();
	}

	@Override
	protected boolean isActivity( final EObject p_object ) {
		return p_object instanceof Activity;
	}
}
