
package de.mdelab.mlsdm.interpreter.debug.ui.launcher.table;

import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;

import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityParameter;
import de.mdelab.mlsdm.ActivityParameterDirectionEnum;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.SDTableActivityParameterData;

/**
 * Class that plays the role of the domain model
 * 
 */
public class MLSDMTableActivityParameterData extends SDTableActivityParameterData<Activity> {
	
	/**
	 * Constructor
	 */
	public MLSDMTableActivityParameterData() {
		super();
	}

	@Override
	public void initData( 	final Activity p_activity,
							final Map<String, String> p_parameters,
							final ResourceSet p_resSet ) {
		clearParameters();
		
		if ( p_activity != null ) {
			for ( final ActivityParameter parameter : p_activity.getParameters() ) {
				if ( parameter.getDirection() == ActivityParameterDirectionEnum.IN || parameter.getDirection() == ActivityParameterDirectionEnum.INOUT ) {
					final String name = parameter.getName();
					
					final String uriStr = p_parameters.get( name );
					final EObject paramValue;
					
					if ( uriStr != null && !uriStr.isEmpty() ) {
						paramValue = p_resSet.getEObject( URI.createURI( uriStr ), true );
					}
					else {
						paramValue = null;
					}
					
					addParameter( new MLSDMTableActivityParameter( name, parameter.getType(), paramValue ) );
				}
			}
		}
	}

	@Override
	public void initData( final Activity p_activity ) {
		clearParameters();
		
		if ( p_activity != null ) {
			for (final ActivityParameter parameter : p_activity.getParameters() ) {
				if ( parameter.getDirection() == ActivityParameterDirectionEnum.IN || parameter.getDirection() == ActivityParameterDirectionEnum.INOUT ) {
					addParameter( new MLSDMTableActivityParameter( parameter.getName(), parameter.getType(), null ) );
				}
			}
		}
	}
}
