package de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.Polyline;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ConnectionNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramWorkbenchPart;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.diagram.part.MlsdmDiagramEditorUtil;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.IMLSDMDebugUiBreakpointConstants;
import de.mdelab.mlstorypatterns.diagram.custom.edit.parts.IMLExpressionEditPart;
import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugTarget;
import de.mdelab.sdm.interpreter.code.debug.ui.storyDiagramEditor.SDEditorManager;
import de.mdelab.sdm.interpreter.core.notifications.NotificationTypeEnum;

public class MLSDMEditorManager extends SDEditorManager<MLElementWithUUID> {
	
	private static final String MODEL_FILE_EXT = "mlsdm";
	
	private static final String DIAG_FILE_EXT = MODEL_FILE_EXT + "_diagram";

	private ChangeFigureTransaction	lastChangeFigureTransaction;

	public MLSDMEditorManager(final SDDebugTarget<MLElementWithUUID, Activity> debugTarget)	{
		super( debugTarget );
	}

	@Override
	public void focusDiagramElement( final MLElementWithUUID p_element ) {

		/*
		 * Open the editor
		 */
		Display.getDefault().syncExec( new Runnable() {
			
			@Override
			public void run() {
//				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				try	{
//					final IEditorPart editorPart = openEditor(editorInput, MLSDMDebuggerStoryDiagramEditor.ID);
//
//					final MLSDMDebuggerStoryDiagramEditor storyDiagramEditor = (MLSDMDebuggerStoryDiagramEditor) editorPart;
//
//					/*
//					 * Get the corresponding element in the diagram editor's
//					 * resource. Note, that the provided element is located in
//					 * the instance graph editor's resource. We have to find the
//					 * corresponding element in the diagram editor's resource
//					 * using the element's UUID.
//					 */
//
//					final Activity activity = (Activity) storyDiagramEditor.getDiagram().getElement();
//
//					final EObject eObject = activity.eResource().getEObject(element.getUuid());
//
//					final EditPart editPart = storyDiagramEditor.getDiagramEditPart().findEditPart( storyDiagramEditor.getDiagramEditPart(), eObject );
					final IDiagramWorkbenchPart editor = openEditor( p_element );
					final EditPart editPart = findEditPart( editor, p_element );
				
					assert editPart != null;
					
					doFocusDiagramElement( editor, editPart );
				}
				catch (final PartInitException p_ex ) {
					throw new RuntimeException( p_ex );
				}
			}
		} );
	}
	
	public static EditPart findEditPart( 	final IDiagramWorkbenchPart p_editor,
											final MLElementWithUUID p_element ) {
		return findEditPart( p_editor, p_element.getUuid() );
	}
	
	public static EditPart findEditPart( 	final IDiagramWorkbenchPart p_editor,
											final String p_elementUuid ) {
		/*
		 * Get the corresponding element in the diagram editor's
		 * resource. Note, that the provided element is located in
		 * the instance graph editor's resource. We have to find the
		 * corresponding element in the diagram editor's resource
		 * using the element's UUID.
		 */
		return findEditPart( p_editor.getDiagramEditPart(), p_elementUuid );
	}
	
	private static EditPart findEditPart( 	final DiagramEditPart p_parentEditPart,
											final String p_semElemUuid ) {
		final EditPart part = findEditPart( (EditPart) p_parentEditPart, p_semElemUuid );
		
		if ( part != null ) {
			return part;
		}
		
		for ( final Object connection : p_parentEditPart.getConnections() ) {
			if ( connection instanceof EditPart ) {
				final EditPart connPart = findEditPart( (EditPart) connection, p_semElemUuid );
				
				if ( connPart != null ) {
					return connPart;
				}
			}
		}
		
		return null;
	}
	
	private static EditPart findEditPart( 	final EditPart p_parentEditPart,
											final String p_semElemUuid ) {
		final MLElementWithUUID elem;
		if ( p_parentEditPart instanceof IMLExpressionEditPart ) {
			elem = ( (IMLExpressionEditPart) p_parentEditPart ).getMLExpression();
		}
		else {
			elem = (MLElementWithUUID) p_parentEditPart.getAdapter( MLElementWithUUID.class );
		}
			
		if ( elem != null && p_semElemUuid.equals( elem.getUuid() ) ) {
			return p_parentEditPart;
		}
		
		for ( final Object child : p_parentEditPart.getChildren() ) {
			if ( child instanceof EditPart ) {
				final EditPart childPart = (EditPart) child;
				
				final EditPart foundPart = findEditPart( childPart, p_semElemUuid );
				
				if ( foundPart != null ) {
					return foundPart;
				}
			}
		}
		
		return null;
	}
	
	private void doFocusDiagramElement( final IDiagramWorkbenchPart p_editorPart,
										final EditPart p_editPart ) {
		@SuppressWarnings("unchecked")
		final List<EditPart> parts = p_editPart == null ? Collections.EMPTY_LIST :  Collections.singletonList( p_editPart );
		
		MlsdmDiagramEditorUtil.selectElementsInDiagram( p_editorPart, parts );
	}
	
	private IDiagramWorkbenchPart openEditor( final MLElementWithUUID p_element )
	throws PartInitException {
		try {
			final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			final MLSDMDebuggerEditorInput editorInput = getEditorInput( p_element );
	
			final IEditorPart part = page.openEditor( editorInput, MLSDMDebuggerStoryDiagramEditor.ID );
			
			return part instanceof IDiagramWorkbenchPart ? (IDiagramWorkbenchPart) part : null; 
		} 
		catch ( final CoreException p_ex ) {
			throw new PartInitException( "", p_ex );
		}
		catch ( final IOException p_ex ) {
			throw new PartInitException( "", p_ex );
		}
	}

	@Override
	public void highlightStoryDiagramElement(	final MLElementWithUUID p_element,
												final NotificationTypeEnum notification ) {
		/*
		 * Open the editor
		 */
		Display.getDefault().syncExec(new Runnable() {
			
			@Override
			public void run() {
//				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
//
				try	{
//					final IEditorPart editorPart = page.openEditor(editorInput, MLSDMDebuggerStoryDiagramEditor.ID);
//
//					final MLSDMDebuggerStoryDiagramEditor storyDiagramEditor = (MLSDMDebuggerStoryDiagramEditor) editorPart;
//
//					/*
//					 * Get the corresponding element in the diagram editor's
//					 * resource. Note, that the provided element is located in
//					 * the instance graph editor's resource. We have to find the
//					 * corresponding element in the diagram editor's resource
//					 * using the element's UUID.
//					 */
//
//					final Activity activity = (Activity) storyDiagramEditor.getDiagram().getElement();
//
//					EObject eObject = activity.eResource().getEObject(element.getUuid());
//
//					EditPart editPart = null;
//
//					if ((eObject instanceof ActivityEdge) || (eObject instanceof AbstractStoryPatternLink))
//					{
//						for (final Object object : storyDiagramEditor.getDiagramEditPart().getConnections())
//						{
//							assert object instanceof ConnectionEditPart;
//
//							if (((View) ((ConnectionEditPart) object).getModel()).getElement() == eObject)
//							{
//								editPart = (IGraphicalEditPart) object;
//								break;
//							}
//						}
//					}
//					else if (eObject instanceof MLExpression)
//					{
//						editPart = (IGraphicalEditPart) storyDiagramEditor.getDiagramEditPart().findEditPart(
//								storyDiagramEditor.getDiagramEditPart(), eObject);
//
//						while (editPart == null)
//						{
//							eObject = eObject.eContainer();
//							editPart = (IGraphicalEditPart) storyDiagramEditor.getDiagramEditPart().findEditPart(
//									storyDiagramEditor.getDiagramEditPart(), eObject);
//						}
//					}
//					else
//					{
//						editPart = (IGraphicalEditPart) storyDiagramEditor.getDiagramEditPart().findEditPart(
//								storyDiagramEditor.getDiagramEditPart(), eObject);
//					}

					//assert editPart != null;
					
//					switch ( notification ) {
//						case STORY_PATTERN_MATCHING_STARTED:
//							stepCounter = 0;
//							
//							break;
//	
//						default:
//							break;
//					}
					
					final IDiagramWorkbenchPart editor = openEditor( p_element );
					final EditPart editPart = findEditPart( editor, p_element );
				
					assert editPart != null;

					/*
					 * Undo the last change figure transaction
					 */
					rollBackLastChangeFigureTransactions();
					
					final ChangeFigureTransaction transaction;
					final Color notifColor = MLSDMEditorManager.this.getColorForNotification( notification );

					/*
					 * if it is a link, make it thicker and color it
					 * 
					 * DB: better check for instances of the generic connection edit part type
					 */
					if ( editPart instanceof ConnectionNodeEditPart ) {
//							editPart instanceof ActivityEdgeEditPart || 
//							editPart instanceof StoryPatternLinkEditPart ||
//							editPart instanceof de.mdelab.mlstorypatterns.diagram.edit.parts.StoryPatternLinkEditPart ||
//							editPart instanceof de.mdelab.mlstorypatterns.diagram.edit.parts.StoryPatternLinkEditPart ||
//							editPart instanceof MapEntryLinkEditPart ) {
						final Polyline polyline = (Polyline) ( (GraphicalEditPart) editPart ).getFigure();
						transaction = new ChangeFigureTransaction( polyline, notifColor, 3 );
					}
					/*
					 * Change the border of nodes
					 */
					else {
						transaction = new ChangeFigureTransaction( ( (AbstractGraphicalEditPart) editPart ).getFigure(), new LineBorder( notifColor, 3 ) );
					}
					
					doFocusDiagramElement( editor, editPart );

					setLastChangeFigureTransaction( transaction );
				}
				catch ( final PartInitException p_ex ) {
					throw new RuntimeException( p_ex );
				}
			}
		} );
	}

	private void setLastChangeFigureTransaction( final ChangeFigureTransaction p_changeFigureTransaction ) {
		lastChangeFigureTransaction = p_changeFigureTransaction;
	}

	private void rollBackLastChangeFigureTransactions() {
		if ( lastChangeFigureTransaction != null ) {
			lastChangeFigureTransaction.rollBack();

			lastChangeFigureTransaction = null;
		}
	}
	
	public void clearHighlighting() {
		rollBackLastChangeFigureTransactions();
	}

	@Override
	public MLSDMDebuggerEditorInput getEditorInput( final MLElementWithUUID p_element )
	throws CoreException, IOException {
		/*
		 * Check if an editor is already open that contains the element
		 */
		EObject elemIter = p_element;

		while ( !( elemIter instanceof Activity) && elemIter != null ) {
			elemIter = elemIter.eContainer();
		}

		assert elemIter instanceof Activity;

		final Activity activity = (Activity) elemIter;

		final URI diagramURI = getDiagramURI( activity );

//		MLSDMDebuggerEditorInput editorInput = null;
		
//		if (activity instanceof MLElementWithUUID) {
		return new MLSDMDebuggerEditorInput( diagramURI, getDebugTarget(), activity.getUuid() );
//		}

//		return editorInput;
	}

	public static URI getModelURI( final URI p_diagUri ) {
		return p_diagUri.trimFileExtension().appendFileExtension( MODEL_FILE_EXT );
	}

	public static URI getDiagramURI( final URI p_modelUri ) 
	throws CoreException, IOException {

		// DB: search for an existing diagram first
		URI diagramURI = p_modelUri.trimFileExtension().appendFileExtension( DIAG_FILE_EXT );
		
		if ( !URIConverter.INSTANCE.exists( diagramURI, null ) ) {
			final IFolder folder = openTempFolder( p_modelUri.trimSegments( 1 ).toPlatformString( true ) );

			diagramURI = URI.createPlatformResourceURI(	folder.getFullPath() + File.separator + p_modelUri.lastSegment(), false ).trimFileExtension().appendFileExtension( DIAG_FILE_EXT );
		}
		
		return diagramURI;
	}
	
	public static URI getDiagramURI( final MLElementWithUUID p_element ) 
	throws CoreException, IOException {
		return getDiagramURI( p_element.eResource().getURI() );
//		URI diagramURI = modelRes.getURI().trimFileExtension().appendFileExtension( DIAG_FILE_EXT );
//
//		if ( !URIConverter.INSTANCE.exists( diagramURI, null ) ) {
//			try {
//				openTempProject();
//			}
//			catch (final CoreException e) {
//				throw new RuntimeException( "Could not open project '" + MLSDMDebugUiConstants.TEMPORARY_STORY_DIAGRAMS_PATH + "'.", e );
//			}
//
//			diagramURI = URI.createPlatformResourceURI(	/*"/" +*/ MLSDMDebugUiConstants.TEMPORARY_STORY_DIAGRAMS_PATH + "/" + p_activity.getUuid() + "." + DIAG_FILE_EXT, false);
//	
//			/*
//			 * Delete the story diagram if it exists but is too old.
//			 */
//			if ( URIConverter.INSTANCE.exists( diagramURI, null ) )	{
//				final Map<String, ?> attributes = URIConverter.INSTANCE.getAttributes(diagramURI, null);
//	
//				final long timestamp = (Long) attributes.get(URIConverter.ATTRIBUTE_TIME_STAMP);
//	
//				if (timestamp < modelRes.getTimeStamp()) {
//					try	{
//						URIConverter.INSTANCE.delete(diagramURI, null);
//					}
//					catch ( final IOException e ) {
//						throw new RuntimeException("Could not delete file '" + diagramURI.toString() + "'.", e);
//					}
//				}
//			}
//		}
//
//		return diagramURI;
	}

	/**
	 * DB: Store temporary diagrams in a folder within the project instead of a global project for the workspace
	 * @param p_projectPath
	 * @return
	 * @throws CoreException
	 */
	private static IFolder openTempFolder( final String p_projectPath ) 
	throws CoreException {
		final IWorkspace workspace = ResourcesPlugin.getWorkspace();
		final IWorkspaceRoot root = workspace.getRoot();
		final IFolder folder = root.getFolder( new Path( p_projectPath + File.separator + IMLSDMDebugUiBreakpointConstants.TEMPORARY_STORY_DIAGRAMS_PATH ) );

		if ( !folder.exists() ) {
			folder.create( true, true, new NullProgressMonitor() );
		}
		
		return folder;
	}

	@Override
	public void closeAllStoryDiagramEditors() {
		Display.getDefault().syncExec(new Runnable() {
			@Override
			public void run() {
				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				final IEditorReference[] editors = page.findEditors(null, MLSDMDebuggerStoryDiagramEditor.ID, IWorkbenchPage.MATCH_ID);

				page.closeEditors(editors, true);
			}
		} );
	}
}
